"""
CemPy Copyright (C) 2008-2009   V. Smilauer
Python frontend to CEMHYD3D cement hydration model v.3.0
CEMHYD3D comes from NIST, Gaithersburg, USA under GNU public licence and was programmed by D.P.Bentz
Extension to low density C-S-H and high density C-S-H and elastic homogenization
done by vit.smilauer@fsv.cvut.cz, 02/2009

CEMHYD3D calculation runs as a thread invoked from GUI interface
Two queues are defined for interchanging data between GUI and thread
class Token is used in the threads
Use quit button otherwise loosing of calculation thread control
GNU Public licence
"""

from Tkinter import *
import Pmw
import tkMessageBox
from Numeric import *
import time
import pylab

import matplotlib

from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
from matplotlib.figure import Figure

import threading
import Queue
import tkFileDialog
import CEM_tools_py
import cemhydmat

#all changes are written from GUI and thread in this one-instance class
class Token:#flags to monitor current processes
  def __init__(self):#constructor
    self.pause = 0
    self.quit = 0
    self.simulation = 0
    self.input_microstructure = 0
    self.cyclingGUI = 0

class CemCalc(threading.Thread):#inherit from threading.Thread class
  def __init__(self, GUI, GUIcemQueue, GUItoken):#constructor
    threading.Thread.__init__(self)
    self.GUIcemQueue = GUIcemQueue#what we get from GUI interface
    self.GUI = GUI#share pointer to MyApp within this class
    self.GUItoken = GUItoken
    self.hydration = 0#hydration process
    self.hasWork = 0

  def __del__(self):#Destructor
    #print("Called destructor")
    None

  #once run() is left, the thread is automatically destroyed
  #run() is called from GUI as a thread
  def run(self):
    #print("In CemCalc().run")
    self.token = self.GUIcemQueue.get()#wait till signal from GUI comes
    if self.token.quit:
      #print 'token quit at CemCalc().run'#if the calculation did not start
      return
    if self.token.simulation and not self.token.pause:
      self.hasWork = 1
      if not self.token.input_microstructure:
        #print("Generate input microstructure")
        self.Ct = CEM_tools_py.CEM_tools_py()
        #convert to lists
        self.CEMHYD_variables = []
        self.CEMHYD_var_concrete = []
        for el in range (0,len(self.GUI.variables)):
          self.CEMHYD_variables.append(self.GUI.variables[el].get())
        for el in range (0,len(self.GUI.var_concrete)):
          self.CEMHYD_var_concrete.append(self.GUI.var_concrete[el].get())
        #print(self.CEMHYD_variables)
        #print(self.GUI.volume)
        #print(self.CEMHYD_var_concrete)
        #call exposed C++ function and pass arrays of variables
        self.Ct.GenerateInputCEMHYD(self.CEMHYD_variables,self.GUI.volume,self.CEMHYD_var_concrete, self.GUI.fin)
        self.Cem = cemhydmat.cemhydmat()
        self.Cem.readcem("CEMHYD_inp_file.src")
        self.GUItoken.input_microstructure = 1

      #print("Running hydration")
      self.hydration = 1
      while True:
        try:
          self.token = self.GUIcemQueue.get_nowait()
        #except NameError:
        #  print "NameError"
        #  pass
        except:
          #print "pass"
          pass
        if self.token.quit:
            #print("Quit in run() routine")
            #self.OutQueueUpdate(token)#to GUI interface
            break

        if self.token.pause:
          #print("Pause in run() routine")
          time.sleep(0.1)#add little time delay not to load processor

        if not self.token.pause:
          #print("Not pause in run() routine")
          #print self.token.quit
          #move one cycle with prescribed temperature (if not adiabatic)
          #t1 = time.clock()
          #time.sleep(0.3)
          #for aaaa in range (0,1000000):
            #None
          self.Cem.MoveCycles(float(self.GUI.variables[14].get()), 1)
          #t2 = time.clock()
          #print t2 - t1


class GUI:
  def __init__(self, parent):#constructor
    self.parent = parent
    self.setupGUI(self.parent)
    self.token = Token()
    self.GUIcemQueue = Queue.Queue()
    self.thread = CemCalc(self,self.GUIcemQueue,self.token)
    self.thread.start()#calls run() method

  #setup GUI face
  def setupGUI(self, parent):
    #constants for controlling layout of buttons

    self.myParent = parent
    #self.myParent.geometry("640x400")
    root.wm_title("CemPy ver 0.11, 04/2009")
    self.top = self.myParent.winfo_toplevel()
    self.top.rowconfigure(0, weight=1)#make stretchable
    self.top.columnconfigure(0, weight=1)#make stretchable

    #Topmost frame is called myContainer1
    self.myContainer1 = Frame(parent)
    self.myContainer1.rowconfigure(0, weight=1)#make stretchable
    self.myContainer1.columnconfigure(0, weight=1)#make stretchable
    self.myContainer1.grid(row=0,column=0,sticky=N+W+E+S)

    #Inside myContainer1, we create left_frame, disp_frame and concrete_frame
    self.left_frame = Frame(self.myContainer1, borderwidth=5)
    self.left_frame.grid(row=0,column=0,sticky=W+N)

    self.disp_frame = Frame(self.myContainer1, borderwidth=5)
    self.disp_frame.rowconfigure(0, weight=1)#make stretchable
    self.disp_frame.columnconfigure(0, weight=1)#make stretchable
    self.disp_frame.columnconfigure(1, weight=1)#make stretchable
    self.disp_frame.grid(row=0,column=1,sticky=N+W+E+S)

    self.concrete_frame = Frame(self.myContainer1, borderwidth=5)
    #self.concrete_frame.rowconfigure(0, weight=1)#make stretchable
    #self.concrete_frame.columnconfigure(0, weight=1)#make stretchable
    #self.concrete_frame.columnconfigure(1, weight=1)#make stretchable
    self.concrete_frame.grid(row=1,column=0,columnspan=2,sticky=N+W)

    #divide left_frame into menu_frame and control_frame
    self.menu_frame = Frame(self.left_frame, borderwidth=5)
    self.menu_frame.grid(row=0,column=0,sticky=W+N)

    self.control_frame = Frame(self.left_frame, borderwidth=5)
    #self.control_frame.rowconfigure(0, weight=1)#make stretchable
    #self.control_frame.columnconfigure(0, weight=1)#make stretchable
    self.control_frame.grid(row=1,column=0,sticky=W+N)

    #control_frame with grid layout method
    Label_names = ["Random seed", "RVE edge size", "Induction time t_0", "Cycle-time mapping", "Blaine fineness [m2/kg]", "Gypsum anhydrate", "Gypsum hemihydrate", "Gypsum dihydrate", "C3S", "C2S", "C3A", "C4AF", "w/c (not w/b)", "Activation energy", "Initial temperature" ]
    Label_def_val = ["100", "10", "2.0", "0.00078", "380", "0.0", "0.0", "5.0", "70.0", "13.0", "9.0","8.0", "0.3", "38.3", "20", "data.dat" ]
    Label_def_units = ["","um", "h", "h", "", "vol. %","vol. %","vol. %","mass %","mass %","mass %","mass %", "-", "kJ/mol", "oC"]
    #declare array of variables to have data from Entry's
    self.variables = []

    #create menubutton with Menu
    self.MenuButton = Menubutton(self.menu_frame, text = "File", relief=RAISED)
    self.MenuButton.grid(row=0,column=0,sticky=W)
    self.MenuButton.menu = Menu(self.MenuButton, tearoff=0)
    self.MenuButton["menu"] = self.MenuButton.menu
    self.MenuButton.menu.add_command(label="Load input data  Ctrl+O", command=self.LoadData)
    self.MenuButton.menu.add_command(label="Save input data Ctrl+S", command=self.SaveData)
    self.MenuButton.menu.add_command(label="Quit                  Ctrl+Q", command=self.quit)

    self.HelpButton = Button(self.menu_frame,text="Help", command = self.help_window)

    #self.HelpButton1 = Menubutton(self.menu_frame, text = "Help", relief=RAISED)
    self.HelpButton.grid(row=0,column=1,sticky=W)
    #self.MenuButton1.menu = Menu(self.MenuButton1)
    #self.MenuButton1["menu"] = self.MenuButton1.menu
    #self.MenuButton1.menu.add_command(label="Kdo dfsdfsdf About do dfsdfsdf Ab ")

    #generate items in control_frame
    for i in range(0,15):
      Label(self.control_frame, text=Label_names[i]).grid(row=i+1,sticky=NW)
      self.variables.append(StringVar())
      self.variables[i].set(Label_def_val[i])
      self.Lbl=Entry(self.control_frame,width=10, textvariable=self.variables[i], bg="white")
      self.Lbl.grid(row=i+1,column=1)
      if i == 12:
        self.Lbl.bind('<KeyPress>',self.concrete_specifications_calc)#from w/c update
        self.Lbl.bind('<Button>',self.concrete_specifications_calc)
      if i== 4:
        self.Fineness = StringVar();
        self.Fineness.set("Dyck");
        self.fin = 0;
        self.Finen = Checkbutton(self.control_frame, textvariable=self.Fineness)
        self.Finen.grid(row=i+1,column=2, sticky=W)
        self.Finen.bind("<Button-1>", self.button_Fine)
        self.Finen.bind("<Return>", self.button_Fine)
        self.Finen.bind("<space>", self.button_Fine)
        continue;
      Label(self.control_frame, text=Label_def_units[i]).grid(row=i+1,column=2,sticky=NW)
      #print i,Label_def_val[i]

    #Adiabatic curing
    i=i+1
    Label(self.control_frame, text="Adiabatic curing").grid(row=i+1,sticky=W)
    self.var = StringVar()
    self.variables.append(self.var)
    self.variables[i].set("Off")
    self.adia = Checkbutton(self.control_frame, textvariable=self.variables[i])
    self.adia.bind("<Button-1>", self.button_adiabatic)
    self.adia.bind("<Return>", self.button_adiabatic)
    self.adia.bind("<space>", self.button_adiabatic)
    self.adia.grid(row=i+1,column=1, sticky=W)

    #Box of autocorrelation functions
    i=i+1
    Label(self.control_frame, text="Autocorrelation triplet").grid(row=i+1,sticky=W)
    self.var = StringVar()
    self.variables.append(self.var)
    self.combobox = Pmw.ComboBox(self.control_frame, entry_width = 7, dropdown=1, scrolledlist_items=("16130s1","16131s1","16132s1","16133s1","16134s1","16148s1","16149s1","16150s1","16155s1","16594s1"))
    self.combobox.selectitem(0)
    self.combobox.grid(row=i+1,column=1, sticky=N+S+E+W)

    #Curing conditions
    i=i+1
    Label(self.control_frame, text="Curing conditions").grid(row=i+1,sticky=W)
    self.var = StringVar()
    self.variables.append(self.var)
    self.variables[i].set("Saturated")
    self.button_curing = Button(self.control_frame,width=10,textvariable=self.variables[i])
    self.button_curing.grid(row=i+1,column=1)
    self.button_curing.bind("<Button-1>", self.button_curing_click)
    self.button_curing.bind("<Return>", self.button_curing_click)
    self.button_curing.bind("<space>", self.button_curing_click)

    #Plot selection, every time you press different button, update plot
    i=i+1
    self.var = StringVar()
    self.variables.append(self.var)
    self.variables[i].set("Time [d]")
    self.Plot_sel1x = Radiobutton (self.control_frame, variable=self.variables[i], value="Time [h]", text="Time [h]", command=self.drawGraph)
    self.Plot_sel1x.grid(row=i+2,column=0,sticky=W)
    self.Plot_sel1x = Radiobutton (self.control_frame, variable=self.variables[i], value="Time [d]", text="Time [d]", command=self.drawGraph)
    self.Plot_sel1x.grid(row=i+3,column=0,sticky=W)
    self.Plot_sel2x = Radiobutton (self.control_frame, variable=self.variables[i], value="Time log [d]", text="Time log [d]", command=self.drawGraph)
    self.Plot_sel2x.grid(row=i+4,column=0,sticky=W)
    self.Plot_sel3x = Radiobutton (self.control_frame, variable=self.variables[i], value="DoH", text="DoH", command=self.drawGraph)
    self.Plot_sel3x.grid(row=i+5,column=0,sticky=W)
    i=i+1
    self.var = StringVar()
    self.variables.append(self.var)
    self.variables[i].set("DoH")
    self.Plot_sel1y = Radiobutton (self.control_frame, variable=self.variables[i], value="DoH", text="DoH", command=self.drawGraph)
    self.Plot_sel1y.grid(row=i+1,column=1,sticky=W)
    self.Plot_sel2y = Radiobutton (self.control_frame, variable=self.variables[i], value="Heat", text="Heat", command=self.drawGraph)
    self.Plot_sel2y.grid(row=i+2,column=1,sticky=W)
    self.Plot_sel3y = Radiobutton (self.control_frame, variable=self.variables[i], value="Temperature", text="Temperature", command=self.drawGraph)
    self.Plot_sel3y.grid(row=i+3,column=1,sticky=W)
    self.Plot_sel4y = Radiobutton (self.control_frame, variable=self.variables[i], value="Young's modulus", text="Young's modulus", command=self.drawGraph)
    self.Plot_sel4y.grid(row=i+4,column=1,sticky=W)
    self.Plot_sel5y = Radiobutton (self.control_frame, variable=self.variables[i], value="Poisson's ratio", text="Poisson's ratio", command=self.drawGraph)
    self.Plot_sel5y.grid(row=i+5,column=1,sticky=W)

    #plot external data from input.dat
    Label(self.control_frame, text="External datafile").grid(row=i+6,column=0,sticky=W)
    self.variables.append(StringVar())
    self.variables[20].set(Label_def_val[15])
    self.Lbl=Entry(self.control_frame,width=10, textvariable=self.variables[20], bg="white")
    self.Lbl.grid(row=i+6,column=1)
    self.variables.append(StringVar())
    self.variables[21].set("Hidden")
    self.chckb = Checkbutton(self.control_frame, width="7",textvariable=self.variables[21])
    self.chckb.bind("<Button-1>", self.external_datafile)
    self.chckb.bind("<Return>", self.external_datafile)
    self.chckb.bind("<space>", self.external_datafile)
    self.chckb.grid(row=i+6,column=2, sticky=W)

    #Run button
    self.var = StringVar()
    self.variables.append(self.var)
    self.variables[22].set("Run simulation")
    self.button_run = Button(self.control_frame, textvariable=self.variables[22])
    self.button_run.focus_force()
    #Clicking by mouse
    self.button_run.bind("<Button-1>", self.button_run_click)
    #Pressing ENTER or SPACE
    self.button_run.bind("<Return>", self.button_run_click)
    self.button_run.bind("<space>", self.button_run_click)
    self.button_run.grid(row=i+7,column=0,sticky=S)

    #Restart button
    self.button_restart = Button(self.control_frame, text="Restart")
    self.button_restart.bind("<Button-1>", self.restart)
    self.button_restart.bind("<Return>", self.restart)
    self.button_restart.bind("<space>", self.restart)
    self.button_restart.grid(row=i+7,column=1,sticky=N+S+E+W)

    #Quit button
    self.button_quit = Button(self.control_frame, text="Quit")
    self.button_quit.bind("<Button-1>", self.quit)
    self.button_quit.bind("<Return>", self.quit)
    self.button_quit.bind("<space>", self.quit)
    self.button_quit.grid(row=i+7,column=2,sticky=N+S+E+W)

    #Display frame - draw graph from matplotlib
    #self.f = Figure(figsize=(5,4.5), dpi=100)
    self.f = Figure(dpi=100)
    self.a = self.f.add_subplot(111)
    self.axes = self.f.get_axes()[0]
    self.f.subplots_adjust(left=0.15, right = 0.9, bottom=0.15, top = 0.9 )
    self.canvas = FigureCanvasTkAgg(self.f, master=self.disp_frame)
    #self.canvas.show()
    self.canvas.get_tk_widget().grid(row=0, sticky=N+W+E+S)
    self.toolbar = NavigationToolbar2TkAgg( self.canvas, self.disp_frame )
    #self.toolbar.update()
    self.toolbar.dynamic_update()
    self.toolbar.grid(row=1, sticky=N+W+E+S)

    #concrete_frame
    Label(self.concrete_frame, text="Mass [kg]").grid(row=0,column=1,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Density [t/m3]").grid(row=0,column=2,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Volume [l]").grid(row=0,column=3,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Young's modulus [GPa]").grid(row=0,column=4,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Poisson's ratio [-]").grid(row=0,column=5,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Average grain diameter [mm]").grid(row=0,column=6,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Cement (clinker + gypsum)").grid(row=1,column=0,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Cement (SCM)").grid(row=2,column=0,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Water").grid(row=3,column=0,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Fine aggregates (FA)").grid(row=4,column=0,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Coarse aggregates (CA)").grid(row=5,column=0,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Inert filler (IF)").grid(row=6,column=0,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Entrained + entrapped air").grid(row=7,column=0,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Total").grid(row=8,column=0,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="").grid(row=9,column=0,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="ITZ thickness [um]").grid(row=10,column=0,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="E modulus\nfraction in ITZ").grid(row=10,column=2,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Cp SCM+FA+CA+IF [J/g/K]").grid(row=10,column=4,padx="1m",sticky=NW)
    Label(self.concrete_frame, text="Cp clinker+gypsum [J/g/K]").grid(row=10,column=6,padx="1m",sticky=NW)
    Concrete_def_val = ["400","20","1100","450","200","3.15","2.2","2.55","2.55","1.1","20","55","55","40","0.2", "0.2", "0.2", "0.2", "2", "20", "30", "20","0.5", "0.84", "0.75"]
    self.var_concrete = []

    cnt=0
    for cols in 1,2,4,5,6:
      if(cols<3):
        rows_list=1,2,4,5,6
      elif(cols<6):
        rows_list=2,4,5,6
      else:
        rows_list=4,5
      for rows in rows_list:
        self.var_concrete.append(StringVar())
        self.var_concrete[cnt].set(Concrete_def_val[cnt])
        self.entry=Entry(self.concrete_frame,width=10,bg="white",textvariable=self.var_concrete[cnt])
        self.entry.grid(row=rows,column=cols)
        self.entry.bind('<KeyPress>',self.concrete_specifications_calc)#update on any key press
        self.entry.bind('<Button>',self.concrete_specifications_calc)#update on any mouse click
        cnt=cnt+1

    #Entrained + entrapped air
    self.var_concrete.append(StringVar())
    self.var_concrete[20].set(Concrete_def_val[20])
    self.entry=Entry(self.concrete_frame,width=10,bg="white",textvariable=self.var_concrete[20])
    self.entry.grid(row=7,column=3)
    self.entry.bind('<KeyPress>',self.concrete_specifications_calc)#update on any key press
    self.entry.bind('<Button>',self.concrete_specifications_calc)#update on any mouse click

    #ITZ thickness
    self.var_concrete.append(StringVar())
    self.var_concrete[21].set(Concrete_def_val[21])
    Entry(self.concrete_frame,width=10,bg="white",textvariable=self.var_concrete[21]).grid(row=10,column=1)

    #Young's modulus reduction in ITZ
    self.var_concrete.append(StringVar())
    self.var_concrete[22].set(Concrete_def_val[22])
    Entry(self.concrete_frame,width=10,bg="white",textvariable=self.var_concrete[22]).grid(row=10,column=3)

    #Cp
    self.var_concrete.append(StringVar())
    self.var_concrete[23].set(Concrete_def_val[23])
    Entry(self.concrete_frame,width=10,bg="white",textvariable=self.var_concrete[23]).grid(row=10,column=5)
    self.var_concrete.append(StringVar())
    self.var_concrete[24].set(Concrete_def_val[24])
    Entry(self.concrete_frame,width=10,bg="white",textvariable=self.var_concrete[24]).grid(row=10,column=7)

    self.concrete_specifications_calc(event=None)

  def button_adiabatic(self,event):
    if self.variables[15].get() == "Off":
       self.variables[15].set("On")
    else:
      self.variables[15].set("Off")

  def button_Fine(self,event):
    if self.Fineness.get() == "Dyck":#NIST database from Dyckerhoff cements
      self.Fineness.set("Breu")#according to van Breugel
      self.fin = 1;
    else:
      self.Fineness.set("Dyck")
      self.fin = 0;

  def external_datafile(self,event):
    if self.variables[21].get() == "Plotted":
      self.variables[21].set("Hidden")
      self.drawGraph()
    else:
      self.variables[21].set("Plotted")
      self.drawGraph()

  def button_curing_click(self,event):
    if self.variables[17].get() == "Saturated":
      self.variables[17].set("Sealed")
    else:
      self.variables[17].set("Saturated")

  def concrete_specifications_calc(self,event):
    #calculate amount of water
    self.water = float(self.var_concrete[0].get()) * float(self.variables[12].get())
    self.label=Label(self.concrete_frame,text="%.4f    " % self.water )
    self.label.grid(row=3,column=1,padx="1m",sticky=W)
    self.label=Label(self.concrete_frame,text=" %.4f    " % 1.0000 )
    self.label.grid(row=3,column=2,padx="1m",sticky=W)
    self.label=Label(self.concrete_frame,text="%.4f    " % self.water )
    self.label.grid(row=3,column=3,padx="1m",sticky=W)

    #calculate and print
    tot_vol = 0.
    cnt=0
    self.volume = []
    for rows in 0,1,3,4,5:
      self.volume.append(StringVar())
      if(self.var_concrete[cnt+5].get()=="" or float(self.var_concrete[cnt+5].get())==0):
        self.volume[cnt] = 0.
      else:
        try:
          self.volume[cnt] = (float(self.var_concrete[cnt].get())/float(self.var_concrete[cnt+5].get()))
        except:
          self.volume[cnt] = 0.

      tot_vol = tot_vol + self.volume[cnt]
      #print self.volume[cnt], tot_vol
      self.label=Label(self.concrete_frame,text="%.4f    " % self.volume[cnt])
      self.label.grid(row=rows+1,column=3,padx="1m",sticky=W)
      cnt = cnt + 1

    try:
      tot_vol = tot_vol + self.water + float(self.var_concrete[20].get())#add water + (entrained + entrapped) air
    except:
      tot_vol = tot_vol + self.water

    try:
      tot_mas = float(self.var_concrete[0].get())+float(self.var_concrete[1].get())+float(self.var_concrete[2].get())+float(self.var_concrete[3].get())+float(self.var_concrete[4].get())+self.water
    except:
      tot_mas = 0.

    Label(self.concrete_frame, text="%.4f    " % tot_mas).grid(row=8,column=1,padx="1m",sticky=W)
    Label(self.concrete_frame, text="%.4f    " % tot_vol).grid(row=8,column=3,padx="1m",sticky=W)
    Label(self.concrete_frame, text="%.4f    " % float(tot_mas/tot_vol)).grid(row=8,column=2,padx="1m",sticky=W)


  def button_run_click(self,event):#pressed Run button
    #print "Pressed run button"
    self.variables[16].set(self.combobox.get())#update combobox
    #check input values
    checklist = self.variables[:15] + self.var_concrete[:25]
    for x in checklist:
      try:
        float(x.get())
      except (ValueError):
        print "Can not convert your input data to numbers for C++ function"
        return

    if(float(self.var_concrete[22].get())<=0 or float(self.var_concrete[22].get())>1):#check ITZ fraction
      print "ITZ fraction must be in the interval (0;1>"
      return

    #change button label
    if self.variables[22].get() == "Run simulation" or self.variables[22].get() == "Continue":
      self.variables[22].set("Pause simulation")
      self.token.simulation = 1
      self.token.pause = 0
    else:
      self.variables[22].set("Continue")#at pause stage
      self.token.pause = 1

    self.GUIcemQueue.put(self.token)
    if(not self.token.cyclingGUI):
      self.CycleGUIAfter()

  def CycleGUIAfter(self):
    self.token.cyclingGUI = 1
    self.parent.after(1000, self.CycleInGUI)#run CycleInGUI every X ms

  def CycleInGUI(self):
    if self.thread.hasWork:
      if self.thread.hydration and self.variables[22].get() == "Pause simulation":
      #only when input microstructure is generated
        self.drawGraph();
    self.CycleGUIAfter()

  def SaveData(self,*ignore):
    filename = tkFileDialog.asksaveasfilename()
    if filename:
      try:
        f = open(filename, 'w')
        self.variables[16].set(self.combobox.get())#update combobox
        for var in (self.variables[:18]):
          #print var.get()
          f.write(var.get())
          f.write('\n')
        f.write(self.variables[20].get())
        f.write('\n')
        f.write(str(self.fin))
        f.write('\n')
        for i in range(0,19):#add 18 spaces for future enhancement
          f.write('\n')
        for var in self.var_concrete[:25]:
          #print var.get()
          f.write(var.get())
          f.write('\n')
        f.close()
      except:
        print "Can not save file"

  def LoadData(self,*ignore):
    filename = tkFileDialog.askopenfilename()
    if filename:
      try:
        f = open(filename, 'r')
        lines=f.readlines()
        f.close()
        for i in range(0,18):
          self.variables[i].set(lines[i].strip())#remove '/n' at the end of lines
        if self.variables[15].get() == "Off":
          self.adia.deselect()
        else:
          self.adia.select()
        self.combobox.selectitem(lines[16].strip())#update combobox
        self.variables[20].set(lines[18].strip())#update external fileneme
        self.fin = int(lines[19].strip())
        if self.fin == 0:
          self.Fineness.set("Dyck")
          self.Finen.deselect()
        else:
          self.Fineness.set("Breu")
          self.Finen.select()

        for i in range(0,25):
          #print lines[18+21+i].strip()
          self.var_concrete[i].set(lines[19+20+i].strip())#remove '/n' at the end of lines
      except:
        print "Can not open file or structure is corrupted"
      self.concrete_specifications_calc(event=None)

  def drawGraph(self):
    self.a.hold(False)#clear graph and axis labels
    scale_x=1

    if self.variables[19].get() == "DoH":
      filedat=open('adiabatic.out','r')
      col_y=3
      title='Degree of hydration'
      ylabel='Degree of hydration [-]'
      if self.variables[18].get() == "DoH":
        col_x=3
      else:
        col_x=1

    elif self.variables[19].get() == "Heat":
      filedat=open('heat.out','r')
      col_y=4
      title='Heat of hydration'
      ylabel='Heat of hydration [J/g of cement]'
      if self.variables[18].get() == "DoH":
        col_x=3
      else:
        col_x=1

    elif self.variables[19].get() == "Temperature":
      filedat=open('adiabatic.out','r')
      col_y=2
      title='Temperature evolution'
      ylabel='Temperature  [oC]'
      if self.variables[18].get() == "DoH":
        col_x=3
      else:
        col_x=1

    elif self.variables[19].get() == "Young's modulus":
      filedat=open('elas.out','r')
      col_y=3
      title="Young's modulus - plain paste, paste, mortar, concrete "
      ylabel="Young's modulus [GPa]"
      if self.variables[18].get() == "DoH":
        col_x=2
      else:
        col_x=1

    elif self.variables[19].get() == "Poisson's ratio":
      filedat=open('elas.out','r')
      col_y=4
      title="Poisson's ratio of cement paste"
      ylabel="Poisson's ratio of cement paste [-]"
      if self.variables[18].get() == "DoH":
        col_x=2
      else:
        col_x=1

    #set xlabel
    if self.variables[18].get() == "Time [h]":
      xlabel='Hydration time [hours]'
      scale_x=1.
    elif self.variables[18].get() == "Time [d]" or self.variables[18].get() == "Time log [d]":
      xlabel='Hydration time [days]'
      scale_x=24.
    else:
      xlabel='Degree of hydration [-]'

    lines=filedat.readlines()
    filedat.close()
    vect_x = []
    vect_y = []
    vect_y1 = []#for plotting elastic moduli
    vect_y2 = []
    vect_y3 = []

    for line in lines:
      sp=line.split()
      try:
        if(float(sp[col_x])/scale_x >= 0.01):#consider positive numbers [h]
          vect_x.append(float(sp[col_x])/scale_x)
          vect_y.append(float(sp[col_y]))
          if (self.variables[19].get() == "Young's modulus"):#plot other scales
            vect_y1.append(float(sp[5]))
            vect_y2.append(float(sp[7]))
            vect_y3.append(float(sp[9]))
          if (self.variables[19].get() == "Poisson's ratio"):#plot other scales
            vect_y1.append(float(sp[6]))
            vect_y2.append(float(sp[8]))
            vect_y3.append(float(sp[10]))
      except:
        pass #line convert was not possible, skip it

    #check external datafile if should be plotted
    vect_ext_x = []
    vect_ext_y = []
    if self.variables[21].get() == "Plotted":
      ext_filedat=open(self.variables[20].get(),'r')
      for ext_line in ext_filedat.readlines():
        sp=ext_line.split()
        try:
          vect_ext_x.append(float(sp[0])/scale_x)
          vect_ext_y.append(float(sp[1]))
        except:
          pass #skip it
      ext_filedat.close()
    #print vect_ext_x, vect_ext_y
    self.axes.cla()


    if (self.variables[19].get() != "Young's modulus" and self.variables[19].get() != "Poisson's ratio"):
      if (self.variables[18].get() == "Time log [d]"):#logplot new lines
        if(self.variables[21].get() == "Plotted"):
          self.a.semilogx(vect_x, vect_y, 'b', vect_ext_x, vect_ext_y, 'rx')
        else:
          self.a.semilogx(vect_x, vect_y, 'b')
      else:
        if(self.variables[21].get() == "Plotted"):
          self.a.plot(vect_x, vect_y, 'b', vect_ext_x, vect_ext_y, 'rx')
        else:
          self.a.plot(vect_x, vect_y, 'b')
    else:#Young's modulus or Poisson's ratio with legend
      if (self.variables[18].get() == "Time log [d]"):#logplot new lines
        if(self.variables[21].get() == "Plotted"):
          self.a.semilogx(vect_x, vect_y, 'b-^', vect_x, vect_y1, 'g-o',  vect_x, vect_y2, 'r-s', vect_x, vect_y3, 'k-d', vect_ext_x, vect_ext_y, 'rx')
        else:
          self.a.semilogx(vect_x, vect_y, 'b-^', vect_x, vect_y1, 'g-o',  vect_x, vect_y2, 'r-s', vect_x, vect_y3, 'k-d')
      else:
        if(self.variables[21].get() == "Plotted"):
          self.a.plot(vect_x, vect_y, 'b-^', vect_x, vect_y1, 'g-o',  vect_x, vect_y2, 'r-s', vect_x, vect_y3, 'k-d', vect_ext_x, vect_ext_y, 'rx')
        else:
          self.a.plot(vect_x, vect_y, 'b-^', vect_x, vect_y1, 'g-o',  vect_x, vect_y2, 'r-s', vect_x, vect_y3, 'k-d')
      if(self.variables[19].get() == "Young's modulus"):
        position = 'lower right'
      else:
        position = 'upper right'
      self.a.legend(('Plain paste','Paste','Mortar','Concrete'), position)
    self.a.set_title(title)
    self.a.set_xlabel(xlabel)
    self.a.set_ylabel(ylabel)
    #print "Vect_x", vect_x
    #self.a.set_ylim((0,1))
    del vect_x[:], vect_y[:], vect_y1[:], vect_y2[:], vect_y3[:], vect_ext_x[:], vect_ext_y[:]
    self.canvas.draw()# redraw the canvas
    #self.canvas.show()

  def restart(self,event):#pressed Restart button
    #print("Pressed Restart button")
    self.token.pause = 1
    self.token.quit = 1
    self.GUIcemQueue.put(self.token)
    #self.CycleInGUI()
    print("Waiting to join the thread - restart")
    self.thread.join()
    #del self.thread
    self.thread = CemCalc(self,self.GUIcemQueue,self.token)#invoke the class again, destructor is called automatically
    self.thread.start()
    self.token.pause = 0
    self.token.quit = 0
    self.token.simulation = 0
    self.token.input_microstructure = 0
    self.cyclingGUI = 0
    self.variables[22].set("Run simulation")

  def quit(self,*ignore):#pressed Quit button or Ctrl+q
    #print("Pressed Quit button")
    #self.token.pause = 1
    self.token.quit = 1
    self.GUIcemQueue.put_nowait(self.token)
    print("Waiting to join the thread")
    self.thread.join()#timeout
    self.parent.quit()

  def help_window(self):
    tkMessageBox.showinfo("Help", "CemPy - GNU public licence \n(c) Vit Smilauer 2008-2009")

root = Tk()
myapp = GUI(root)
root.bind( '<Control-KeyPress-o>', myapp.LoadData)#Open file - bind to all widgets
root.bind( '<Control-KeyPress-O>', myapp.LoadData)
root.bind( '<Control-KeyPress-s>', myapp.SaveData)
root.bind( '<Control-KeyPress-S>', myapp.SaveData)
root.bind( '<Control-KeyPress-q>', myapp.quit)
root.bind( '<Control-KeyPress-Q>', myapp.quit)
root.mainloop()
